/* xshowgif.c:
 *
 * generic image loader for X11
 *
 * richard j. greco 04.27.90
 * jim frost 09.27.89
 *
 * Copyright 1990, Richard J. Greco.
 * Copyright 1989, 1990 Jim Frost.  See included file "copyright.h" for
 * complete copyright information.
 */

#include "copyright.h"
#include "xshowgif.h"
#include "patchlevel"
#include "gif.h"
#include <errno.h>

/* options array and definitions.  note that the enum values must be in the
 * same order as the options strings.
 */

char *Options[] = {
  "border", /* global options */
  "display",
  "fullscreen",
  "geometry",
  "help",
  "quiet",
  "verbose",
  "version",

  "background", /* image options */
  "brighten",
  "clip",
  "colors",
  "dither",
  "foreground",
  "halftone",
  "name",
  "xzoom",
  "yzoom",
  "zoom",
  NULL
};

enum {

  /* global options
   */

  BORDER= 0, DISPLAY, FULLSCREEN, GEOMETRY, HELP,
  QUIET, VERBOSE, VER_NUM,

  /* local options
   */

  BACKGROUND, BRIGHT, CLIP, COLORS, DITHER, FOREGROUND,
  HALFTONE, NAME, XZOOM, YZOOM, ZOOM
};

extern Image *gifLoad();

Display *Disp = NULL;
int Scrn = 0;

/* the real thing
 */

main(argc, argv)
     int argc;
     char *argv[];
{
  char         *border;
  char         *dname;
  unsigned int  fullscreen;
  unsigned int  verbose;
  unsigned int quiet;
  Image        *dispimage;      /* image that will be sent to the display */
  Display      *disp;           /* display we're sending to */
  int           scrn;           /* screen we're sending to */
  XColor        xcolor;         /* color for border option */
  ImageOptions  images[MAXIMAGES]; /* list of image w/ options to load */
  int           a;
  unsigned int  imagecount;     /* number of images in ImageName array */
  unsigned int  winx, winy;     /* location of window */
  unsigned int  winwidth, winheight; /* geometry of window */
  char   fullname[BUFSIZ];
  ZFILE *zf;


  if (argc < 2)
    usage(argv[0]);

  /* defaults and other initial settings */

  quiet = 0;
  verbose = 0;
  border= NULL;
  dname= NULL;
  fullscreen= 0;
  winx= winy= winwidth= winheight= 0;

  imagecount= 0;
  for (a= 0; a < MAXIMAGES; a++) {
    images[a].name= NULL;
    images[a].bright= 0;
    images[a].clipx= images[a].clipy= 0;
    images[a].clipw= images[a].cliph= 0;
    images[a].dither= 0;
    images[a].colors= 0;
    images[a].fg= images[a].bg= NULL;
    images[a].xzoom= images[a].yzoom= 0;
  }
  for (a= 1; a < argc; a++) {
    switch (optionNumber(argv[a], Options)) {
    case OPT_BADOPT:
      printf("%s: Bad option\n", argv[a]);
      usage(argv[0]);
      /* NOTREACHED */

    case OPT_NOTOPT:
      if (imagecount == MAXIMAGES)
	printf("%s: Too many images (ignoring)\n", argv[++a]);
      else
	images[imagecount++].name= argv[a];
      break;

    case OPT_SHORTOPT:
      printf("%s: Not enough characters to identify option\n", argv[a]);
      usage(argv[0]);
      /* NOTREACHED */

    /* process options global to everything
     */

    case BORDER:
      border= argv[++a];
      break;

    case DISPLAY:
      dname= argv[++a];
      break;

    case GEOMETRY:
      XParseGeometry(argv[++a], &winx, &winy, &winwidth, &winheight);
      break;

    case HELP:
      usage(argv[0]);
      exit(0);

    case QUIET:
      quiet=1;
      verbose= 0;
      break;

    case VERBOSE:
      quiet=0;
      verbose= 1;
      break;

    case VER_NUM:
      printf("xshowgif version %s patchlevel %s by Jim Frost and Rich Greco\n",
	     VERSION, PATCHLEVEL);
      break;

    /* process options local to an image
     */

    case BACKGROUND:
      images[imagecount].bg= argv[++a];
      break;

    case BRIGHT:
      images[imagecount].bright= atoi(argv[++a]);
      break;

    case CLIP:
      if (sscanf(argv[++a], "%d,%d,%d,%d",
		 &images[imagecount].clipx, &images[imagecount].clipy,
		 &images[imagecount].clipw, &images[imagecount].cliph) != 4) {
	printf("Bad argument to -clip\n");
	usage(argv[0]);
	/* NOTREACHED */
      }
      break;

    case COLORS:
      images[imagecount].colors= atoi(argv[++a]);
      if (images[imagecount].colors < 2) {
	printf("Argument to -colors is too low (ignored)\n");
	images[imagecount].colors= 0;
      }
      else if (images[imagecount].colors > 65536) {
	printf("Argument to -colors is too high (ignored)\n");
	images[imagecount].colors= 0;
      }
      break;

    case DITHER:
      images[imagecount].dither= 1;
      break;

    case FOREGROUND:
      images[imagecount].fg= argv[++a];
      break;

    case HALFTONE:
      images[imagecount].dither= 2;
      break;

    case NAME:
      if (imagecount == MAXIMAGES)
	printf("%s: Too many images (ignoring)\n", argv[++a]);
      else
	images[imagecount++].name= argv[++a];
      break;

    case XZOOM:
      images[imagecount].xzoom= atoi(argv[++a]);
      break;

    case YZOOM:
      images[imagecount].yzoom= atoi(argv[++a]);
      break;

    case ZOOM:
      images[imagecount].xzoom= images[imagecount].yzoom= atoi(argv[++a]);
      break;

    default:

      /* this should not happen!
       */

      printf("%s: Internal error parsing arguments\n", argv[0]);
      exit(1);
    }
  }

  if (!imagecount) /* NO-OP from here on */
    exit(0);

  /* start talking to the display
   */

  if (! (Disp= disp= XOpenDisplay(dname))) {
    printf("%s: Cannot open display\n", XDisplayName(dname));
    exit(1);
  }
  Scrn= scrn= DefaultScreen(disp);
  XSetIOErrorHandler(ioErrorHandler);

  dispimage= NULL;

  /* load in each named image
   */

  for (a= 0; a < imagecount; a++) {
    int first_image_in_file;

    first_image_in_file = 1;

    if (findImage(images[a].name, fullname) < 0) {
      if (errno == ENOENT)
	printf("%s: image not found\n", images[a].name);
      else
	perror(fullname);
      continue;
    }

    if (! (zf= zopen(fullname)))
      continue;
    if (gifin_open_file(zf) != GIFIN_SUCCESS) {
      printf("%s: is not a GIF file\n", fullname);
      gifin_close_file();
      continue;
    }

    while((gifin_open_image() == GIFIN_SUCCESS)) {

      if (!quiet && first_image_in_file == 1) {
	tellAboutImage(images[a].name);
	first_image_in_file = 0;
      }

      if ( !(dispimage = gifLoad(zf, fullname, images[a].name)) ) {
	printf("%s: is a corrupted GIF file\n", fullname);
	gifin_close_image();
	gifin_close_file();
	continue;
      }

      if (!images[a].dither &&
	  (BITMAPP(dispimage) || (DefaultDepth(disp, scrn) == 1)))
	images[a].dither= 2;
      dispimage = processImage(disp, scrn, dispimage, &images[a], verbose);

      switch(imageInWindow(disp, scrn, dispimage, winx, winy, winwidth,
			   winheight, fullscreen, verbose)) {
      case '\0': /* window got nuked by someone */
	XCloseDisplay(disp);
	exit(1);

      case '\003':
      case 'q':  /* user quit */
      case 'Q':  /* user quit */
	XCloseDisplay(disp);
	exit(0);

      case 'N':  /* next file */
        gifin_close_image();
        freeImage(dispimage);
        dispimage= NULL;
	goto next_gif_file;

      case 'n':  /* next image */
	break;

      case 'P':  /* previous file */
	if (a > 0)
	  a -= 2;
	else
	  a--;
	goto next_gif_file;
      }

      gifin_close_image();
      freeImage(dispimage);
      dispimage= NULL;
    }
next_gif_file:
    gifin_close_file();
    zclose(zf);
  }

  cleanUpWindow(disp);
  XCloseDisplay(disp);
}
